@@ -1,22 +1,22 @@
 #
-# $Id: Split.pm,v 0.37 2004/01/03 21:31:27 st.schubiger Exp $
+# $Id: Split.pm,v 0.38 2004/01/04 22:03:23 sts Exp $
 
 package Dir::Split;
 
+our $VERSION = '0.38';
+
 use 5.6.1;
 use strict;
 use warnings;
 
-use Carp;
 use File::Copy 'cp';
 use File::Path;
 use File::Spec;
 
-our $VERSION = '0.37';
-
 our (# external data
+        @exists,
+        %failure,
         %track,
-        %warn,
 
      # data
         @Files,
@@ -27,10 +27,15 @@ our (# external data
         $Suffix,
 );
 
-# return codes
-our $ACTION    = 1;
-our $NO_ACTION = 0;
-our $EXISTS   = -1;
+sub ACTION { 1 }
+sub NO_ACTION { 0 }
+sub EXISTS { -1 }
+sub FAILURE { -2 }
+
+sub croak {
+    require Carp;
+    &Carp::croak;
+}
 
 =head1 NAME
 
@@ -42,19 +47,18 @@ Dir::Split - Split files of a directory to subdirectories.
 
  %options = (   mode    =>    'num',
 
-                source  =>    '/tmp/source',
-                target  =>    '/tmp/target',
+                source  =>    '/source',
+                target  =>    '/target',
 
                 options => {  verbose      =>         1,
-                              warn         =>     'all',
-                              override     =>    'none',
+                              override     =>         0,
                 },
                 sub_dir => {  identifier   =>     'sub',
                               file_limit   =>         2,
                               file_sort    =>       '+',
                 },
                 suffix  => {  separator    =>       '-',
-                              continue     =>       'y',
+                              continue     =>         1,
                               length       =>         5,
                 },
  );
@@ -64,20 +68,15 @@ Dir::Split - Split files of a directory to subdirectories.
 
  $return = $dir->split;
 
- $dir->{'source'} = '/tmp/dir1';
- $dir->{'target'} = '/tmp/dir2';
-
- $return = $dir->split;
-
 =head1 DESCRIPTION
 
 C<Dir::Split> moves files to either numbered or characteristic subdirectories.
 
 =head1 CONSTRUCTOR
 
-=head2 new ( \%options )
+=head2 new
 
-Creates an object. C<%options> contains the key/value pairs which will influence the splitting process.
+Creates an object. C<%options> contains the key / value pairs which will influence the splitting process.
 
  $dir = Dir::Split->new (\%options);
 
@@ -109,7 +108,6 @@ sub _tie_var {
                     target  =>    'target',
 
                     options => {  verbose     =>      'verbose',
-                                  warn        =>         'warn',
                                   override    =>     'override',
                     },
                     sub_dir => {  identifier  =>        'ident',
@@ -122,7 +120,6 @@ sub _tie_var {
                                   case        =>         'case',
                     },
     );
-
     my %assigned;
     foreach my $key (keys %$Opt_ref) {
         if (ref $$Opt_ref{$key} eq 'HASH') {
@@ -132,7 +129,6 @@ sub _tie_var {
         }
         else { $assigned{$assign{$key}} = $$Opt_ref{$key} }
     }
-
     return \%assigned;
 };
 
@@ -144,6 +140,12 @@ Split files to subdirectories.
 
  $return = $dir->split;
 
+It is of tremendous importance to notice that checking the return code is a B<must>.
+Leaving the return code untouched will not allow appropriate gathering of harmless
+debug data (such as existing files) and system operations that failed. C<Dir::Split>
+does only report verbose output of mkpath to STDOUT. See I<OPTIONS / debug> on how to
+become aware of existing files and failed system operations (I<copy> & I<unlink>).
+
 B<RETURN CODES>
 
 =over 4
@@ -152,17 +154,21 @@ B<RETURN CODES>
 
 Files moved successfully.
 
-Implies access to C<%Dir::Split::track> (see OPTIONS / tracking).
-
 =item (0)
 
 No action.
 
 =item (-1)
 
-Existing directories/files found.
+Exists.
 
-Implies access to C<%Dir::Split::track> (see OPTIONS / tracking).
+I<(see OPTIONS / debug / existing)>.
+
+=item (-2)
+
+Failure.
+
+I<(see OPTIONS / debug / failures)>.
 
 =back
 
@@ -176,9 +182,9 @@ sub split {
 
     # files found, split.
     if (@Files) {
-        $Ret_Status = $ACTION;
+        $Ret_Status = ACTION;
 
-        $o->_init_presets();
+        #$o->_init_presets(); # unused atm.
 
         # engine
         $o->_sort_files() if $o->{'mode'} eq 'num';
@@ -186,8 +192,8 @@ sub split {
         $o->$sub_suffix();
         $o->_move();
     }
-    # no files? goodbye.
-    else { $Ret_Status = $NO_ACTION }
+    # no files? exit.
+    else { $Ret_Status = NO_ACTION }
 
     return $Ret_Status;
 }
@@ -205,7 +211,6 @@ sub _sanity_input {
                      source      =>    'No source dir specified.',
                      target      =>    'No target dir specified.',
                      verbose     =>    'No verbosity specified.',
-                     warn        =>    'No warn mode specified.',
                      override    =>    'No override mode specified.',
                      ident       =>    'No subdir identifier specified.',
                      sep         =>    'No suffix separator specified.',
@@ -217,7 +222,7 @@ sub _sanity_input {
     );
 
     my $err_input;
-    {
+    {   # generic opts
         unless ($o->{'mode'} eq 'num' || $o->{'mode'} eq 'char') {
             $err_input = $err_msg{'mode'}; last;
         }
@@ -230,10 +235,7 @@ sub _sanity_input {
         unless ($o->{'verbose'} =~ /^0|1$/) {
             $err_input = $err_msg{'verbose'}; last;
         }
-        unless ($o->{'warn'} =~ /^none|file|dir|all$/) {
-            $err_input = $err_msg{'warn'}; last;
-        }
-        unless ($o->{'override'} =~ /^none|file|dir|all$/) {
+        unless ($o->{'override'} =~ /^0|1$/) {
             $err_input = $err_msg{'override'}; last;
         }
         unless ($o->{'ident'} =~ /\w/) {
@@ -242,21 +244,22 @@ sub _sanity_input {
         unless ($o->{'sep'}) {
             $err_input = $err_msg{'sep'}; last;
         }
+        # numeric opts
         if ($o->{'mode'} eq 'num') {
-            unless ($o->{'f_limit'} =~ /\d/) {
+            unless ($o->{'f_limit'} > 0) {
                 $err_input = $err_msg{'f_limit'}; last;
             }
             unless ($o->{'f_sort'} eq '+' || $o->{'f_sort'} eq '-') {
                 $err_input = $err_msg{'f_sort'}; last;
             }
-            unless ($o->{'num_contin'} eq 'y' || $o->{'num_contin'} eq 'n') {
+            unless ($o->{'num_contin'} =~ /^0|1$/) {
                 $err_input = $err_msg{'num_contin'}; last;
             }
-            unless ($o->{'length'} =~ /\d/) {
+            unless ($o->{'length'} > 0) {
                 $err_input = $err_msg{'length'}; last;
             }
-
         }
+        # characteristic opts
         elsif ($o->{'mode'} eq 'char') {
             unless ($o->{'case'} eq 'lower' || $o->{'case'} eq 'upper') {
                 $err_input = $err_msg{'case'}; last;
@@ -287,15 +290,8 @@ sub _gather_files {
 # Initializes var presets.
 #
 
-sub _init_presets {
-    my %Presets = (  warn =>  {  dir   =>    "exists (d)\t",
-                                 file  =>    "exists (f)\t",
-                     },
-    );
-
-    $warn{'dir'} ||= $Presets{'warn'}{'dir'};
-    $warn{'file'} ||= $Presets{'warn'}{'file'};
-}
+#sub _init_presets {
+#}
 
 #
 # _sort_files()
@@ -327,15 +323,14 @@ sub _sort_files {
 sub _suffix_num {
     my $o = $_[0];
 
-    $o->_suffix_num_contin() if $o->{'num_contin'} eq 'y';
+    $o->_suffix_num_contin() if $o->{'num_contin'} == 1;
     $o->_suffix_num_sum_up();
 }
 
 #
 # _suffix_num_contin()
 #
-# Evaluates the highest existing subdir suffix number
-# from the target directory.
+# Evaluates the highest existing subdir suffix number.
 #
 
 sub _suffix_num_contin {
@@ -347,7 +342,6 @@ sub _suffix_num_contin {
 
     # surpress warnings
     $Suffix = 0;
-
     my $sep = quotemeta $o->{'sep'};
     foreach (@dirs) {
         # extract exist. identifier
@@ -369,9 +363,7 @@ sub _suffix_num_contin {
 sub _suffix_num_sum_up {
     my $o = $_[0];
 
-    # initialize suffix or avoid suffix collision if != 0
     $Suffix++;
-
     if (length $Suffix < $o->{'length'}) {
         my $format = "%0.$o->{'length'}".'d';
         $Suffix = sprintf $format, $Suffix;
@@ -413,7 +405,6 @@ sub _move {
     $track{'target'}{'dirs'} = 0;
     $track{'target'}{'files'} = 0;
 
-    # sub handler
     my $sub_move = "_move_$o->{'mode'}";
     $o->$sub_move();
 }
@@ -474,8 +465,6 @@ sub _dir_read {
 # _mkpath ($suffix)
 #
 # Creates subdirs.
-# Upon dir collision optional warning
-# and overriding avoidance.
 #
 
 sub _mkpath {
@@ -483,10 +472,9 @@ sub _mkpath {
 
     $Path = File::Spec->catfile($o->{'target'}, "$o->{'ident'}$o->{'sep'}$suffix");
 
-    return if $o->_exists_and_not_override('dir', $Path);
-
-    mkpath $Path, $o->{'verbose'}
-      or croak qq~Could not create subdir $Path: $!~;
+    return if -e $Path;
+    mkpath $Path, $o->{'verbose'} or
+      die "Directory $Path could not be created: $!\n";
 
     $track{'target'}{'dirs'}++;
 }
@@ -495,8 +483,7 @@ sub _mkpath {
 # _cp_unlink ($file)
 #
 # Copies files and unlinks them.
-# Upon file collision optional warning
-# and overriding avoidance.
+# Upon existing files / failures debug data.
 #
 
 sub _cp_unlink {
@@ -505,11 +492,21 @@ sub _cp_unlink {
     my $path_full = File::Spec->catfile($Path, $file);
     $file = File::Spec->catfile($o->{'source'}, $file);
 
-    return if $o->_exists_and_not_override('file', $path_full);
+    if ($o->_exists_and_not_override($path_full)) {
+        push @exists, $path_full;
+        return;
+    }
 
-    cp $file, $path_full or
-      croak qq~Could not copy file $file to subdir $path_full: $!~;
-    unlink $file or croak qq~Could not remove file $file: $!~;
+    unless (cp $file, $path_full) {
+        push @{$failure{'copy'}}, $path_full;
+        $Ret_Status = FAILURE unless $Ret_Status eq FAILURE;
+        return;
+    }
+    unless (unlink $file) {
+        push @{$failure{'unlink'}}, $file;
+        $Ret_Status = FAILURE unless $Ret_Status eq FAILURE;
+        return;
+    }
 
     $track{'target'}{'files'}++;
 }
@@ -517,21 +514,19 @@ sub _cp_unlink {
 #
 # _exists_and_not_override ($item, $path)
 #
-# Checks for existing files/paths and warns accordingly.
+# Looks out for existing files.
 #
 
 sub _exists_and_not_override {
-    my ($o, $item, $path) = @_;
+    my ($o, $path) = @_;
 
     if (-e $path) {
-        if ($o->{'warn'} eq 'all' || $o->{'warn'} eq $item) {
-            no strict 'refs';
-            warn "$warn{$item} $path\n";
+        if ($o->{'override'} != 1) {
+            if ($Ret_Status ne FAILURE && $Ret_Status ne EXISTS) {
+                $Ret_Status = EXISTS;
+            }
+            return 1;
         }
-
-        $Ret_Status = $EXISTS unless $Ret_Status eq $EXISTS;
-
-        return 1 unless $o->{'override'} eq 'all' || $o->{'override'} eq $item;
     }
 
     return 0;
@@ -566,23 +561,24 @@ Split files to subdirectories with a numeric suffix.
 
     %options = (  mode    =>    'num',
 
-                  source  =>    '/tmp/source',
-                  target  =>    '/tmp/target',
+                  source  =>    '/source',
+                  target  =>    '/target',
 
                   options => {  verbose     =>         1,
-                                warn        =>     'all',
-                                override    =>    'none',
+                                override    =>         0,
                   },
                   sub_dir => {  identifier  =>     'sub',
                                 file_limit  =>         2,
                                 file_sort   =>       '+',
                   },
                   suffix  => {  separator   =>       '-',
-                                continue    =>       'y',
+                                continue    =>         1,
                                 length      =>         5,
                   },
     );
 
+B<options> (mandatory)
+
 =over 4
 
 =over 4
@@ -595,11 +591,11 @@ Split files to subdirectories with a numeric suffix.
 
 =item B<source>
 
-(s) - the source directory.
+(s) - source directory.
 
 =item B<target>
 
-(s) - the target directory.
+(s) - target directory.
 
 =item B<options / verbose>
 
@@ -610,25 +606,13 @@ subdirectories.
    1  enabled
    0  disabled
 
-=item B<options / warn>
-
-(s) - warnings upon the encounter of existing files/dirs.
-
- LEVELS
-   none
-   file
-   dir
-   all
-
 =item B<options / override>
 
-(s) - overriding of existing files/dirs.
+(i) - overriding of existing files.
 
- LEVELS
-   none
-   file
-   dir
-   all
+ MODES
+   1  enabled
+   0  disabled
 
 =item B<sub_dir / identifier>
 
@@ -652,15 +636,16 @@ subdirectories.
 
 =item B<suffix / continue>
 
-(c) - numbering continuation.
+(i) - numbering continuation.
 
  MODES
-   y  yes
-   n  no     (will start at 1)
+   1  enabled
+   0  disabled    (will start at 1)
 
-If numbering continuation is set to yes, and numeric subdirectories are found
-within the target directory which match the given identifier and separator,
-then the suffix numbering will be continued.
+If numbering continuation is enabled, and numeric subdirectories are found
+within target directory which match the given identifier and separator,
+then the suffix numbering will be continued. Disabling numbering continuation
+may cause interfering with existing files.
 
 =item B<suffix / length>
 
@@ -676,17 +661,16 @@ of the highest suffix number.
 =head2 characteristic
 
 Split files to subdirectories with a characteristic suffix. Files
-are assigned to subdirectories which suffixes correspond with the leading character
-of the filenames.
+are assigned to subdirectories which suffixes equal the leading character
+of their filenames.
 
     %options = (  mode    =>    'char',
 
-                  source  =>    '/tmp/source',
-                  target  =>    '/tmp/target',
+                  source  =>    '/source',
+                  target  =>    '/target',
 
                   options => {  verbose     =>         1,
-                                warn        =>     'all',
-                                override    =>    'none',
+                                override    =>         0,
                   },
                   sub_dir => {  identifier  =>     'sub',
                   },
@@ -696,6 +680,8 @@ of the filenames.
 
     );
 
+B<options> (mandatory)
+
 =over 4
 
 =over 4
@@ -708,11 +694,11 @@ of the filenames.
 
 =item B<source>
 
-(s) - the source directory.
+(s) - source directory.
 
 =item B<target>
 
-(s) - the target directory.
+(s) - target directory.
 
 =item B<options / verbose>
 
@@ -723,25 +709,13 @@ subdirectories.
    1  enabled
    0  disabled
 
-=item B<options / warn>
-
-(s) - warnings upon the encounter of existing files/dirs.
-
- LEVELS
-   none
-   file
-   dir
-   all
-
 =item B<options / override>
 
-(s) - overriding of existing files/dirs.
+(i) - overriding of existing files.
 
- LEVELS
-   none
-   file
-   dir
-   all
+ MODES
+   1  enabled
+   0  disabled
 
 =item B<sub_dir / identifier>
 
@@ -763,18 +737,12 @@ subdirectories.
 
 =back
 
-=head2 warnings
-
-The warning messages upon the encounter of existing directories/files;
-paths of existing directories/files will be applied after the messages.
-
-    %Dir::Split::warn = (  dir   =>    "exists (d)\t",
-                           file  =>    "exists (f)\t",
-    );
-
 =head2 tracking
 
-Keeps track of how many files the source and directories/files the target consists of.
+C<%Dir::Split::track> keeps count of how many files the source and directories / files
+the target consists of. It may prove its usefulness, if the amount of files that could
+not be transferred due to existing ones has to be counted.
+Each time a new splitting is attempted, the track will be reseted.
 
     %Dir::Split::track = (  source  =>    {  files  =>    512,
                             },
@@ -785,22 +753,34 @@ Keeps track of how many files the source and directories/files the target consis
 
 Above example: directory consisting 512 files successfully splitted to 128 directories.
 
-The B<override> option will depending upon your operating system and/or file permissions
-not always be able to override existing directories/files.
-In case C<split()> returns a EXISTS (implying existing directories/files), following code
-will print the amount of files that could not be moved.
+=head2 debug
 
-    my $not_moved = $Dir::Split::track{'source'}{'files'} -
-                    $Dir::Split::track{'target'}{'files'};
+=head3 existing
 
-    warn "Could not move $not_moved files.\n";
+If C<split()> returns a I<EXISTS>, this implys that the B<override> option is disabled and
+files could not be moved due to existing files within the target subdirectories; they will have
+their paths appearing in the according keys in C<%Dir::Split::exists>.
 
-Currently there is no possibility to access the paths of directories/files
-that could not be moved.
+   file    @Dir::Split::exists    # existing files, not attempted to
+                                  # be overwritten.
+
+=head3 failures
+
+If C<split()> returns a I<FAILURE>, this most often implys that the B<override> option is enabled
+and existing files could not be overriden. Files that could not be copied / unlinked,
+will have their paths appearing in the according keys in C<%Dir::Split::failure>.
+
+   file    @{$Dir::Split::failure{'copy'}}      # files that couldn't be copied,
+                                                # most often on overriding failures.
+
+           @{$Dir::Split::failure{'unlink'}}    # files that could be copied but not unlinked,
+                                                # rather seldom.
+
+It is highly recommended to evaluate those arrays on I<FAILURE>.
 
 =head1 EXAMPLES
 
-Assuming I</tmp/source> contains 5 files:
+Assuming I</source> contains 5 files:
 
     +- _12.tmp
     +- abc.tmp
@@ -808,7 +788,7 @@ Assuming I</tmp/source> contains 5 files:
     +- ghi.tmp
     +- jkl.tmp
 
-After splitting the directory tree in I</tmp/target> will look as following:
+After splitting the directory tree in I</target> will look as following:
 
 =head2 numeric splitting
 
@@ -838,14 +818,11 @@ After splitting the directory tree in I</tmp/target> will look as following:
 
 C<Perl 5.6.1>; C<File::Copy>, C<File::Path>, C<File::Spec>.
 
-=head1 CAVEATS & BUGS
+=head1 CAVEATS
 
 Recursive source directory processing is not supported;
 existing directories within the source directory will be ignored.
 
-Minus integer values in B<file_limit> and B<length> options
-cause an odd behavior at runtime.
-
 =head1 SEE ALSO
 
 perl(1)
